@@ -394,10 +394,16 @@ create_local_queue (const char *name,
 	 q = (remote_printer_t *)cupsArrayNext(remote_printers))
       if (!strcasecmp(q->name, p->name))
 	break;
-    p->duplicate = (q && q->status != STATUS_DISAPPEARED) ? 1 : 0;
+    p->duplicate = (q && q->status != STATUS_DISAPPEARED &&
+		    q->status != STATUS_UNCONFIRMED) ? 1 : 0;
     if (p->duplicate)
       debug_printf("cups-browsed: Printer %s already available through host %s.\n",
 		   p->name, q->host);
+    else if (q) {
+      q->duplicate = 1;
+      debug_printf("cups-browsed: Unconfirmed/disappeared printer %s already available through host %s, marking that printer duplicate of the newly found one.\n",
+		   p->name, q->host);
+    }
   } else {
     /* Non-CUPS printer broadcasts are most probably from printers
        directly connected to the network and using the IPP protocol.
@@ -1009,7 +1015,7 @@ void generate_local_queue(const char *host,
 #ifdef HAVE_AVAHI
     /* If the remote queue has a PPD file, the "product" field of the
        TXT record is populated. If it has no PPD file the remote queue
-       is a raw queue and so we do not jknow enough about the printer
+       is a raw queue and so we do not know enough about the printer
        behind it for auto-creating a local queue pointing to it. */
     int raw_queue = 0;
     if (txt) {
@@ -1022,11 +1028,11 @@ void generate_local_queue(const char *host,
 	}
       } else
 	raw_queue = 1;
-    } else
+    } else if (domain && domain[0] != '\0')
       raw_queue = 1;
     if (raw_queue) {
       /* The remote CUPS queue is raw, ignore it */
-      debug_printf("cups-browsed: Remote CUPS queue %s on host %s is raw, ignored.\n",
+      debug_printf("cups-browsed: Remote Bonjour-advertised CUPS queue %s on host %s is raw, ignored.\n",
 		   remote_queue, remote_host);
       free (remote_host);
       return;
